#!/usr/bin/env perl

use strict;

my $fileformats = $ARGV[0];
$fileformats = "./src/jalview/io/FileFormats.java" unless $fileformats;

# default mimetype will be text/x-$shortname
# TODO: find an actual extension for mat, see JAL-Xxxxx for outstanding issues too
# TODO: look up standard mime type used for BLASTfmt matrices, etc
my $mimetypes = {
  rnaml => "application/rnaml+xml",
  biojson => "application/x-jalview-biojson+json",
  jnet => "application/x-jalview-jnet+text",
  features => "application/x-jalview-features+text",
  scorematrix => "application/x-jalview-scorematrix+text",
  pdb => "chemical/x-pdb",
  mmcif => "chemical/x-cif",
  mmcif2 => "chemical/x-mcif",
  jalview => "application/x-jalview+xml+zip",
  jvl => "application/x-jalview-jvl+text",
  annotations => "application/x-jalview-annotations+text",
};

my @dontaddshortname = qw(features json);
my @dontaddextension = qw(html xml json jar mfa fastq);
my $add_associations = {
  biojson => {shortname=>"biojson",name=>"BioJSON",extensions=>["biojson"]},
  gff2 => {shortname=>"gff2",name=>"Generic Features Format v2",extensions=>["gff2"]},
  gff3 => {shortname=>"gff3",name=>"Generic Features Format v3",extensions=>["gff3"]},
  features => {shortname=>"features",name=>"Jalview Features",extensions=>["features","jvfeatures"]},
  annotations => {shortname=>"annotations",name=>"Jalview Annotations",extensions=>["annotations","jvannotations"]},
  mmcif2 => {shortname=>"mmcif2",name=>"mmCIF",extensions=>["mcif","mmcif"]},
  jvl => {shortname=>"jvl",name=>"Jalview Version Locator",extensions=>["jvl"],iconfile=>"Jalview-Version-Locator"},
  jnet => {shortname=>"jnet",name=>"JnetFile",extensions=>["concise","jnet"]},
  scorematrix => {shortname=>"scorematrix",name=>"Substitution Matrix",extensions=>["mat"]},
};
my $add_extensions = {
  blc => ["blc"],
};

my $mactemplatefile = "file_associations_template-Info_plist.xml";
my $i4jtemplatefile = "file_associations_template-install4j.xml";
my $mactemplate;
my $i4jtemplate;
open(MT,"<$mactemplatefile") or dir("Could not open '$mactemplatefile' for reading");
while(<MT>){
  $mactemplate .= $_;
}
close(MT);
open(IT,"<$i4jtemplatefile") or dir("Could not open '$i4jtemplatefile' for reading");
while(<IT>){
  $i4jtemplate .= $_;
}
close(IT);
my $macauto;
my $i4jauto;

my $macautofile = $mactemplatefile;
$macautofile =~ s/template/auto$1/;
my $i4jautofile = $i4jtemplatefile;
$i4jautofile =~ s/template/auto$1/;

open(MA,">$macautofile") or die ("Could not open '$macautofile' for writing");
print MA "<key>CFBundleDocumentTypes</key>\n<array>\n\n";
open(IA,">$i4jautofile") or die ("Could not open '$i4jautofile' for writing");

open(IN, "<$fileformats") or die ("Could not open '$fileformats' for reading");
my $id = 10000;
my $file_associations = {};
while(my $line = <IN>) {
  $line =~ s/\s+/ /g;
  $line =~ s/(^ | $)//g;
  if ($line =~ m/^(\w+) ?\( ?"([^"]*)" ?, ?"([^"]*)" ?, ?(true|false) ?, ?(true|false) ?\)$/i) {
    my $shortname = lc($1);
    next if (grep($_ eq $shortname, @dontaddshortname));
    my $name = $2;
    my $extensions = $3;
    $extensions =~ s/\s+//g;
    my @possextensions = split(m/,/,$extensions);
    my @extensions;
    my $addext = $add_extensions->{$shortname};
    if (ref($addext) eq "ARRAY") {
      push(@possextensions, @$addext);
    }
    for my $possext (@possextensions) {
      next if grep($_ eq $possext, @extensions);
      next if grep($_ eq $possext, @dontaddextension);
      push(@extensions,$possext);
    }
    next unless scalar(@extensions);
    $file_associations->{$shortname} = {
      shortname => $shortname,
      name => $name,
      extensions => \@extensions
    };
  }
}
close(IN);

my %all_associations = (%$file_associations, %$add_associations);

for my $fa (sort keys %all_associations) {
  my $shortname = $fa;
  my $a = $all_associations{$fa};
  my $name = $a->{name};
  my $extensions = $a->{extensions};
  my $mimetype = $mimetypes->{$shortname};
  $mimetype = "text/x-$shortname" unless $mimetype;

  my $iconfile = $a->{iconfile};
  $iconfile = "Jalview-File" unless $iconfile;

  my @extensions = @$extensions;
  #warn("LINE: $line\nFound extensions '".join("', '", @extensions)."' for $name Files ($shortname)'n");

  my $macentry = $mactemplate;
  my $i4jentry = $i4jtemplate;

  $macentry =~ s/\$\$NAME\$\$/$name/g;
  $macentry =~ s/\$\$SHORTNAME\$\$/$shortname/g;
  $macentry =~ s/\$\$MIMETYPE\$\$/$mimetype/g;
  $macentry =~ s/\$\$ICONFILE\$\$/$iconfile/g;

  $i4jentry =~ s/\$\$NAME\$\$/$name/g;
  $i4jentry =~ s/\$\$SHORTNAME\$\$/$shortname/g;
  $i4jentry =~ s/\$\$MIMETYPE\$\$/$mimetype/g;
  $i4jentry =~ s/\$\$ICONFILE\$\$/$iconfile/g;

  while ($macentry =~ m/\$\$([^\$]*)EXTENSIONS([^\$]*)\$\$/) {
    my $pre = $1;
    my $post = $2;
    my $macextensions;
    for my $ext (@extensions) {
      $macextensions .= $pre.$ext.$post;
    }
    $macentry =~ s/\$\$${pre}EXTENSIONS${post}\$\$/$macextensions/g;
  }
  print MA $macentry;

  for my $ext (@extensions) {
    my $i4jextentry = $i4jentry;
    $i4jextentry =~ s/\$\$EXTENSION\$\$/$ext/g;
    $i4jextentry =~ s/\$\$ID\$\$/$id/g;
    $id++;

    print IA $i4jextentry;
  }
}

close(IA);
print MA "</array>\n";
close(MA);
